@@ -158,8 +158,8 @@ def cluster_inspect(identifiers):
     has_exception = False
     for identifier in identifiers:
         try:
-            app = utils.fetch_remote_cluster(identifier)
-            print(json.dumps(tutum.Cluster.fetch(app.uuid).get_all_attributes(), indent=2))
+            cluster = utils.fetch_remote_cluster(identifier)
+            print(json.dumps(tutum.Cluster.fetch(cluster.uuid).get_all_attributes(), indent=2))
         except Exception as e:
             print(e, file=sys.stderr)
             has_exception = True
@@ -171,8 +171,8 @@ def cluster_logs(identifiers):
     has_exception = False
     for identifier in identifiers:
         try:
-            app = utils.fetch_remote_cluster(identifier)
-            print(app.logs)
+            cluster = utils.fetch_remote_cluster(identifier)
+            print(cluster.logs)
         except Exception as e:
             print(e, file=sys.stderr)
             has_exception = True
@@ -184,9 +184,10 @@ def cluster_open():
     try:
         cluster_list = tutum.Cluster.list()
         deployed_datetimes = {}
-        for app in cluster_list:
-            if app.web_public_dns and app.state in ["Running", "Partly running"]:
-                deployed_datetimes[utils.from_utc_string_to_utc_datetime(app.deployed_datetime)] = app.web_public_dns
+        for cluster in cluster_list:
+            if cluster.web_public_dns and cluster.state in ["Running", "Partly running"]:
+                deployed_datetimes[
+                    utils.from_utc_string_to_utc_datetime(cluster.deployed_datetime)] = cluster.web_public_dns
         if deployed_datetimes:
             max_datetime = max(deployed_datetimes.keys())
             webbrowser.open("http://" + deployed_datetimes[max_datetime])
@@ -199,18 +200,19 @@ def cluster_open():
 
 def cluster_ps(quiet=False, status=None):
     try:
-        headers = ["NAME", "UUID", "STATUS", "IMAGE", "SIZE (#)", "DEPLOYED", "WEB HOSTNAME"]
+        headers = ["NAME", "UUID", "STATUS", "IMAGE", "DEPLOYED", "WEB HOSTNAME"]
         cluster_list = tutum.Cluster.list(state=status)
         data_list = []
         long_uuid_list = []
-        for app in cluster_list:
-            data_list.append([app.unique_name, app.uuid[:8], utils.add_unicode_symbol_to_state(app.state),
-                              app.image_name, "%s (%d)" % (app.container_size, app.current_num_containers),
-                              utils.get_humanize_local_datetime_from_utc_datetime_string(app.deployed_datetime),
-                              app.web_public_dns])
-            long_uuid_list.append(app.uuid)
+        for cluster in cluster_list:
+            data_list.append([cluster.unique_name, cluster.uuid[:8],
+                              utils.add_unicode_symbol_to_state(cluster.state),
+                              cluster.image_name,
+                              utils.get_humanize_local_datetime_from_utc_datetime_string(cluster.deployed_datetime),
+                              cluster.web_public_dns])
+            long_uuid_list.append(cluster.uuid)
         if len(data_list) == 0:
-            data_list.append(["", "", "", "", "", "", ""])
+            data_list.append(["", "", "", "", "", ""])
 
         if quiet:
             for uuid in long_uuid_list:
@@ -227,10 +229,10 @@ def cluster_redeploy(identifiers, tag):
     has_exception = False
     for identifier in identifiers:
         try:
-            app = utils.fetch_remote_cluster(identifier)
-            result = app.redeploy(tag)
+            cluster = utils.fetch_remote_cluster(identifier)
+            result = cluster.redeploy(tag)
             if result:
-                print(app.uuid)
+                print(cluster.uuid)
         except Exception as e:
             print(e, file=sys.stderr)
             has_exception = True
@@ -238,24 +240,25 @@ def cluster_redeploy(identifiers, tag):
         sys.exit(EXCEPTION_EXIT_CODE)
 
 
-def cluster_run(image, name, container_size, target_num_containers, run_command, entrypoint, container_ports,
-            container_envvars, linked_to_cluster, linked_to_container, autorestart, autoreplace, autodestroy,
-            roles, sequential, web_public_dns):
+def cluster_run(image, name, cpu_shares, memory, memory_swap, target_num_containers, run_command, entrypoint,
+                container_ports, container_envvars, linked_to_cluster, linked_to_container, autorestart, autoreplace,
+                autodestroy, roles, sequential, web_public_dns):
     try:
         ports = utils.parse_ports(container_ports)
         envvars = utils.parse_envvars(container_envvars)
-        links_cluster = utils.parse_links(linked_to_cluster, 'to_application')
+        links_cluster = utils.parse_links(linked_to_cluster, 'to_cluster')
         links_container = utils.parse_links(linked_to_container, 'to_container')
-        app = tutum.Cluster.create(image=image, name=name, container_size=container_size,
+        cluster = tutum.Cluster.create(image=image, name=name, cpu_shares=cpu_shares,
+                                       memory=memory, memory_swap=memory_swap,
                                        target_num_containers=target_num_containers, run_command=run_command,
                                        entrypoint=entrypoint, container_ports=ports, container_envvars=envvars,
-                                       linked_to_application=links_cluster, linked_to_container=links_container,
+                                       linked_to_cluster=links_cluster, linked_to_container=links_container,
                                        autorestart=autorestart, autoreplace=autoreplace, autodestroy=autodestroy,
                                        roles=roles, sequential_deployment=sequential, web_public_dns=web_public_dns)
-        app.save()
-        result = app.start()
+        cluster.save()
+        result = cluster.start()
         if result:
-            print(app.uuid)
+            print(cluster.uuid)
     except Exception as e:
         print(e, file=sys.stderr)
         sys.exit(EXCEPTION_EXIT_CODE)
@@ -265,11 +268,11 @@ def cluster_scale(identifiers, target_num_containers):
     has_exception = False
     for identifier in identifiers:
         try:
-            app = utils.fetch_remote_cluster(identifier)
-            app.target_num_containers = target_num_containers
-            result = app.save()
+            cluster = utils.fetch_remote_cluster(identifier)
+            cluster.target_num_containers = target_num_containers
+            result = cluster.save()
             if result:
-                print(app.uuid)
+                print(cluster.uuid)
         except Exception as e:
             print(e, file=sys.stderr)
             has_exception = True
@@ -300,10 +303,10 @@ def cluster_start(identifiers):
     has_exception = False
     for identifier in identifiers:
         try:
-            app = utils.fetch_remote_cluster(identifier)
-            result = app.start()
+            cluster = utils.fetch_remote_cluster(identifier)
+            result = cluster.start()
             if result:
-                print(app.uuid)
+                print(cluster.uuid)
         except Exception as e:
             print(e, file=sys.stderr)
             has_exception = True
@@ -315,10 +318,10 @@ def cluster_stop(identifiers):
     has_exception = False
     for identifier in identifiers:
         try:
-            app = utils.fetch_remote_cluster(identifier)
-            result = app.stop()
+            cluster = utils.fetch_remote_cluster(identifier)
+            result = cluster.stop()
             if result:
-                print(app.uuid)
+                print(cluster.uuid)
         except Exception as e:
             print(e, file=sys.stderr)
             has_exception = True
@@ -330,10 +333,10 @@ def cluster_terminate(identifiers):
     has_exception = False
     for identifier in identifiers:
         try:
-            app = utils.fetch_remote_cluster(identifier)
-            result = app.delete()
+            cluster = utils.fetch_remote_cluster(identifier)
+            result = cluster.delete()
             if result:
-                print(app.uuid)
+                print(cluster.uuid)
         except Exception as e:
             print(e, file=sys.stderr)
             has_exception = True
@@ -345,8 +348,8 @@ def container_inspect(identifiers):
     has_exception = False
     for identifier in identifiers:
         try:
-            app = utils.fetch_remote_container(identifier)
-            print(json.dumps(tutum.Container.fetch(app.uuid).get_all_attributes(), indent=2))
+            container = utils.fetch_remote_container(identifier)
+            print(json.dumps(tutum.Container.fetch(container.uuid).get_all_attributes(), indent=2))
         except Exception as e:
             print(e, file=sys.stderr)
             has_exception = True
@@ -369,15 +372,15 @@ def container_logs(identifiers):
 
 def container_ps(cluster_identifier, quiet=False, status=None):
     try:
-        headers = ["NAME", "UUID", "STATUS", "IMAGE", "RUN COMMAND", "SIZE", "EXIT CODE", "DEPLOYED", "PORTS"]
+        headers = ["NAME", "UUID", "STATUS", "IMAGE", "RUN COMMAND", "EXIT CODE", "DEPLOYED", "PORTS"]
 
         if cluster_identifier is None:
             containers = tutum.Container.list(state=status)
         elif utils.is_uuid4(cluster_identifier):
-            containers = tutum.Container.list(application__uuid=cluster_identifier, state=status)
+            containers = tutum.Container.list(cluster__uuid=cluster_identifier, state=status)
         else:
-            containers = tutum.Container.list(application__unique_name=cluster_identifier, state=status) + \
-                         tutum.Container.list(application__uuid__startswith=cluster_identifier, state=status)
+            containers = tutum.Container.list(cluster__unique_name=cluster_identifier, state=status) + \
+                         tutum.Container.list(cluster__uuid__startswith=cluster_identifier, state=status)
 
         data_list = []
         long_uuid_list = []
@@ -392,14 +395,17 @@ def container_ps(cluster_identifier, quiet=False, status=None):
                 ports.append(ports_string)
 
             ports_string = ", ".join(ports)
-            data_list.append([container.unique_name, container.uuid[:8],
-                              utils.add_unicode_symbol_to_state(container.state), container.image_name,
-                              container.run_command, container.container_size, container.exit_code,
+            data_list.append([container.unique_name,
+                              container.uuid[:8],
+                              utils.add_unicode_symbol_to_state(container.state),
+                              container.image_name,
+                              container.run_command,
+                              container.exit_code,
                               utils.get_humanize_local_datetime_from_utc_datetime_string(container.deployed_datetime),
                               ports_string])
             long_uuid_list.append(container.uuid)
         if len(data_list) == 0:
-            data_list.append(["", "", "", "", "", "", "", "", ""])
+            data_list.append(["", "", "", "", "", "", "", ""])
         if quiet:
             for uuid in long_uuid_list:
                 print(uuid)
@@ -425,20 +431,21 @@ def container_redeploy(identifiers, tag):
         sys.exit(EXCEPTION_EXIT_CODE)
 
 
-def container_run(image, name, container_size, run_command, entrypoint, container_ports,
-            container_envvars, linked_to_cluster, linked_to_container, autorestart, autoreplace, autodestroy,
-            roles, web_public_dns):
+def container_run(image, name, cpu_shares, memory, memory_swap, run_command, entrypoint, container_ports,
+                  container_envvars, linked_to_cluster, linked_to_container, autorestart, autoreplace, autodestroy,
+                  roles, web_public_dns):
     try:
         ports = utils.parse_ports(container_ports)
         envvars = utils.parse_envvars(container_envvars)
-        links_cluster = utils.parse_links(linked_to_cluster, 'to_application')
+        links_cluster = utils.parse_links(linked_to_cluster, 'to_cluster')
         links_container = utils.parse_links(linked_to_container, 'to_container')
-        container = tutum.Container.create(image=image, name=name, container_size=container_size,
-                                       run_command=run_command,
-                                       entrypoint=entrypoint, container_ports=ports, container_envvars=envvars,
-                                       linked_to_application=links_cluster, linked_to_container=links_container,
-                                       autorestart=autorestart, autoreplace=autoreplace, autodestroy=autodestroy,
-                                       roles=roles, web_public_dns=web_public_dns)
+        container = tutum.Container.create(image=image, name=name, cpu_shares=cpu_shares,
+                                           memory=memory, memory_swap=memory_swap,
+                                           run_command=run_command,
+                                           entrypoint=entrypoint, container_ports=ports, container_envvars=envvars,
+                                           linked_to_cluster=links_cluster, linked_to_container=links_container,
+                                           autorestart=autorestart, autoreplace=autoreplace, autodestroy=autodestroy,
+                                           roles=roles, web_public_dns=web_public_dns)
         container.save()
         result = container.start()
         if result:
@@ -542,7 +549,7 @@ def image_push(name, public):
         print('Pushing %s to public registry ...' % repository)
 
         output_status = NO_ERROR
-        #tag a image to its name to check if the images exists
+        # tag a image to its name to check if the images exists
         try:
             docker_client.tag(name, name)
         except Exception as e:
@@ -589,7 +596,7 @@ def image_push(name, public):
             sys.exit(TUTUM_AUTH_ERROR_EXIT_CODE)
 
         try:
-            registry = os.getenv('TUTUM_REGISTRY_URL') or 'https://r.tutum.co/v1/'
+            registry = os.getenv('TUTUM_REGISTRY_URL') or 'https://tutum.co/v1/'
             docker_client.login(user, apikey, registry=registry)
         except Exception as e:
             print(e, file=sys.stderr)
@@ -675,3 +682,256 @@ def image_update(repositories, username, password, description):
                 print(image.name)
         except Exception as e:
             print(e, file=sys.stderr)
+
+
+def node_list(quiet):
+    try:
+        headers = ["UUID", "FQDN", "LASTSEEN", "STATUS", "CLUSTER"]
+        node_list = tutum.Node.list()
+        data_list = []
+        long_uuid_list = []
+        for node in node_list:
+            cluster_name = node.node_cluster
+            try:
+                cluster_name = tutum.NodeCluster.fetch(node.node_cluster.strip("/").split("/")[-1]).name
+            except:
+                pass
+
+            data_list.append([node.uuid[:8],
+                              node.external_fqdn,
+                              utils.get_humanize_local_datetime_from_utc_datetime_string(node.last_seen),
+                              node.state,
+                              cluster_name])
+            long_uuid_list.append(node.uuid)
+        if len(data_list) == 0:
+            data_list.append(["", "", "", "", ""])
+        if quiet:
+            for uuid in long_uuid_list:
+                print(uuid)
+        else:
+            utils.tabulate_result(data_list, headers)
+    except Exception as e:
+        print(e, file=sys.stderr)
+        sys.exit(EXCEPTION_EXIT_CODE)
+
+
+def node_inspect(identifiers):
+    has_exception = False
+    for identifier in identifiers:
+        try:
+            node = utils.fetch_remote_node(identifier)
+            print(json.dumps(tutum.Node.fetch(node.uuid).get_all_attributes(), indent=2))
+        except Exception as e:
+            print(e, file=sys.stderr)
+            has_exception = True
+    if has_exception:
+        sys.exit(EXCEPTION_EXIT_CODE)
+
+
+def node_rm(identifiers):
+    has_exception = False
+    for identifier in identifiers:
+        try:
+            node = utils.fetch_remote_node(identifier)
+            result = node.delete()
+            if result:
+                print(node.uuid)
+        except Exception as e:
+            print(e, file=sys.stderr)
+            has_exception = True
+    if has_exception:
+        sys.exit(EXCEPTION_EXIT_CODE)
+
+
+def nodecluster_list(quiet):
+    try:
+        headers = ["NAME", "UUID", "REGION", "TYPE", "DEPLOYED", "STATUS", "CURRENT#NODES", "TARGET#NODES"]
+        nodecluster_list = tutum.NodeCluster.list()
+        data_list = []
+        long_uuid_list = []
+        for nodecluster in nodecluster_list:
+            if quiet:
+                long_uuid_list.append(nodecluster.uuid)
+                continue
+
+            node_type = nodecluster.node_type
+            region = nodecluster.region
+            try:
+                node_type = tutum.NodeType.fetch(nodecluster.node_type.strip("/").split("/")[-1]).label
+                region = tutum.Region.fetch(nodecluster.region.strip("/").split("/")[-1]).label
+            except:
+                pass
+
+            data_list.append([nodecluster.name,
+                              nodecluster.uuid[:8],
+                              region,
+                              node_type,
+                              utils.get_humanize_local_datetime_from_utc_datetime_string(nodecluster.deployed_datetime),
+                              nodecluster.state,
+                              nodecluster.current_num_nodes,
+                              nodecluster.target_num_nodes])
+            long_uuid_list.append(nodecluster.uuid)
+        if len(data_list) == 0:
+            data_list.append(["", "", "", "", "", "", "", ""])
+        if quiet:
+            for uuid in long_uuid_list:
+                print(uuid)
+        else:
+            utils.tabulate_result(data_list, headers)
+    except Exception as e:
+        print(e, file=sys.stderr)
+        sys.exit(EXCEPTION_EXIT_CODE)
+
+
+def nodecluster_inspect(identifiers):
+    has_exception = False
+    for identifier in identifiers:
+        try:
+            nodecluster = utils.fetch_remote_nodecluster(identifier)
+            print(json.dumps(tutum.NodeCluster.fetch(nodecluster.uuid).get_all_attributes(), indent=2))
+        except Exception as e:
+            print(e, file=sys.stderr)
+            has_exception = True
+    if has_exception:
+        sys.exit(EXCEPTION_EXIT_CODE)
+
+
+def nodecluster_show_providers(quiet):
+    try:
+        headers = ["ID", "NAME", "REGIONS"]
+        data_list = []
+        name_list = []
+        provider_list = tutum.Provider.list()
+        for provider in provider_list:
+            if quiet:
+                name_list.append(provider.name)
+                continue
+
+            region_label = []
+            for region in provider.regions:
+                try:
+                    region_obj = tutum.Region.fetch(region.strip("/").split("/")[-1])
+                    region_label.append(region_obj.label or region_obj.name)
+                except:
+                    pass
+            data_list.append([provider.id, provider.name, ", ".join(region_label)])
+
+        if len(data_list) == 0:
+            data_list.append(["", "", ""])
+        if quiet:
+            for name in name_list:
+                print(name)
+        else:
+            utils.tabulate_result(data_list, headers)
+    except Exception as e:
+        print(e, file=sys.stderr)
+        sys.exit(EXCEPTION_EXIT_CODE)
+
+
+def nodecluster_show_regions(provider_id):
+    try:
+        provider = tutum.Provider.fetch(str(provider_id))
+        provider_name = provider.name
+    except Exception as e:
+        if "Status 404" in e.message:
+            print("The id(%d) of provider does not exist!" % provider_id, file=sys.stderr)
+            sys.exit(EXCEPTION_EXIT_CODE)
+        print(e, file=sys.stderr)
+        sys.exit(EXCEPTION_EXIT_CODE)
+    print('Listing all available regions of provider: %s ...' % provider_name)
+
+    try:
+        headers = ["ID", "NAME", "TYPE"]
+        data_list = []
+        region_list = tutum.Region.list(provider=provider_id)
+        for region in region_list:
+            node_type_list = []
+            for node_type in region.node_types:
+                try:
+                    obj = tutum.NodeType.fetch(node_type.strip("/").split("/")[-1])
+                    node_type_list.append(obj.label or obj.name)
+                except:
+                    pass
+            data_list.append([region.id, region.label or region.name, ", ".join(node_type_list)])
+
+        if len(data_list) == 0:
+            data_list.append(["", "", ""])
+        utils.tabulate_result(data_list, headers)
+    except Exception as e:
+        print(e, file=sys.stderr)
+        sys.exit(EXCEPTION_EXIT_CODE)
+
+
+def nodecluster_show_types(region_id):
+    try:
+        region = tutum.Region.fetch(str(region_id))
+        region_name = region.label or region.name
+    except Exception as e:
+        if "Status 404" in e.message:
+            print("The ID(%d) of region does not exist! " % region_id, file=sys.stderr)
+            sys.exit(EXCEPTION_EXIT_CODE)
+        print(e, file=sys.stderr)
+        sys.exit(EXCEPTION_EXIT_CODE)
+    print('Listing all available types of region: %s ...' % region_name)
+
+    try:
+        headers = ["ID", "TYPE"]
+        data_list = []
+        type_list = tutum.NodeType.list(region=region_id)
+        for node_type in type_list:
+            data_list.append([node_type.resource_uri.strip("/").split("/")[-1], node_type.label or node_type.name])
+        if len(data_list) == 0:
+            data_list.append(["", "", ""])
+        utils.tabulate_result(data_list, headers)
+    except Exception as e:
+        print(e, file=sys.stderr)
+        sys.exit(EXCEPTION_EXIT_CODE)
+
+
+def nodecluster_create(target_num_nodes, name, provider_id, region_id, nodetype_id):
+    provider = "/api/v1/provider/%d/" % provider_id
+    region = "/api/v1/region/%d/" % region_id
+    nodetype = "/api/v1/nodetype/%d/" % nodetype_id
+
+    try:
+        nodecluster = tutum.NodeCluster.create(name=name, target_num_nodes=target_num_nodes,
+                                               provider=provider, region=region, node_type=nodetype)
+        nodecluster.save()
+        result = nodecluster.deploy()
+        if result:
+            print(nodecluster.uuid)
+    except Exception as e:
+        print(e, file=sys.stderr)
+        sys.exit(EXCEPTION_EXIT_CODE)
+
+
+def nodecluster_rm(identifiers):
+    has_exception = False
+    for identifier in identifiers:
+        try:
+            nodecluster = utils.fetch_remote_nodecluster(identifier)
+            result = nodecluster.delete()
+            if result:
+                print(nodecluster.uuid)
+        except Exception as e:
+            print(e, file=sys.stderr)
+            has_exception = True
+    if has_exception:
+        sys.exit(EXCEPTION_EXIT_CODE)
+
+
+
+def nodecluster_scale(identifiers, target_num_containers):
+    has_exception = False
+    for identifier in identifiers:
+        try:
+            nodecluster = utils.fetch_remote_nodecluster(identifier)
+            nodecluster.target_num_nodes = target_num_containers
+            result = nodecluster.save()
+            if result:
+                print(nodecluster.uuid)
+        except Exception as e:
+            print(e, file=sys.stderr)
+            has_exception = True
+    if has_exception:
+        sys.exit(EXCEPTION_EXIT_CODE)
