@@ -19,29 +19,29 @@ def add_build_parser(subparsers):
 def add_cluster_parser(subparsers):
     # tutum cluster
     cluster_parser = subparsers.add_parser('cluster', help='Cluster-related operations',
-                                        description='Cluster-related operations')
+                                           description='Cluster-related operations')
     cluster_subparser = cluster_parser.add_subparsers(title='tutum cluster commands', dest='subcmd')
 
     # tutum cluster alias
     alias_parser = cluster_subparser.add_parser('alias',
-                                             help="Set a custom FQDN (CNAME) to a running web cluster",
-                                             description="Set a custom DNS record (CNAME) to a running web cluster")
+                                                help="Set a custom FQDN (CNAME) to a running web cluster",
+                                                description="Set a custom DNS record (CNAME) to a running web cluster")
     alias_parser.add_argument('identifier', help="cluster's UUID (either long or short) or name", nargs='+')
     alias_parser.add_argument('dns', help='custom FQDN to use for this web cluster')
 
     # tutum cluster inspect
     inspect_parser = cluster_subparser.add_parser('inspect', help="Get all details from an cluster",
-                                               description="Get all details from an cluster")
+                                                  description="Get all details from an cluster")
     inspect_parser.add_argument('identifier', help="cluster's UUID (either long or short) or name", nargs='+')
 
     # tutum cluster logs
     logs_parser = cluster_subparser.add_parser('logs', help='Get logs from an cluster',
-                                            description='Get logs from an cluster')
+                                               description='Get logs from an cluster')
     logs_parser.add_argument('identifier', help="cluster's UUID (either long or short) or name", nargs='+')
 
     # tutum cluster open
     cluster_subparser.add_parser('open', help='Open last web cluster launched',
-                              description='Open last web cluster launched', )
+                                 description='Open last web cluster launched', )
 
     # tutum cluster ps
     ps_parser = cluster_subparser.add_parser('ps', help='List clusters', description='List clusters')
@@ -51,20 +51,20 @@ def add_cluster_parser(subparsers):
 
     # tutum cluster redeploy
     redeploy_parser = cluster_subparser.add_parser('redeploy', help='Redeploy a running cluster with a '
-                                                                 'new version/tag',
-                                                description='Redeploy a running cluster with a new version/tag')
+                                                                    'new version/tag',
+                                                   description='Redeploy a running cluster with a new version/tag')
     redeploy_parser.add_argument('identifier', help="cluster's UUID (either long or short) or name", nargs='+')
     redeploy_parser.add_argument('-t', '--tag', help='tag of the image to redeploy')
 
     # tutum cluster run
     run_parser = cluster_subparser.add_parser('run', help='Create and run a new cluster',
-                                           description='Create and run a new cluster', )
+                                              description='Create and run a new cluster', )
     run_parser.add_argument('image', help='the name of the image used to deploy this cluster')
     run_parser.add_argument('-n', '--name', help='a human-readable name for the cluster '
                                                  '(default: image_tag without namespace)')
-    run_parser.add_argument('-s', '--container-size', help='the size of the cluster containers '
-                                                           '(default: XS, possible values: XS, S, M, L, XL)',
-                            default='XS')
+    run_parser.add_argument('--cpushares', help='Relative weight for CPU Shares', type=int)
+    run_parser.add_argument('--memory', help='RAM memory hard limit in MB', type=int)
+    run_parser.add_argument('--memoryswap', help='Memory swap hard limit in MB', type=int)
     run_parser.add_argument('-t', '--target-num-containers',
                             help='the number of containers to run for this cluster (default: 1)', type=int,
                             default=1)
@@ -98,15 +98,16 @@ def add_cluster_parser(subparsers):
 
     # tutum cluster scale
     scale_parser = cluster_subparser.add_parser('scale', help='Scale a running cluster',
-                                                 description='Scale a running cluster', )
+                                                description='Scale a running cluster', )
     scale_parser.add_argument('identifier', help="cluster's UUID (either long or short) or name", nargs='+')
     scale_parser.add_argument("target_num_containers", metavar="target-num-containers",
-                                  help="target number of containers to scale this cluster to", type=int)
+                              help="target number of containers to scale this cluster to", type=int)
     # tutum cluster set
-    set_parser = cluster_subparser.add_parser('set', help='Enable or disable Crash Recovery and Autodestroy features to '
-                                                       'an existing cluster',
-                                           description='Enable or disable Crash Recovery and Autodestroy features to '
-                                                       'an existing cluster')
+    set_parser = cluster_subparser.add_parser('set',
+                                              help='Enable or disable Crash Recovery and Autodestroy features to '
+                                                   'an existing cluster',
+                                              description='Enable or disable Crash Recovery and Autodestroy features to'
+                                                          ' an existing cluster')
     set_parser.add_argument('identifier', help="cluster's UUID (either long or short) or name", nargs='+')
     set_parser.add_argument('--autorestart', help="whether the containers should be restarted if they stop "
                                                   "(default: OFF)", choices=['OFF', 'ON_FAILURE', 'ALWAYS'])
@@ -118,34 +119,34 @@ def add_cluster_parser(subparsers):
 
     # tutum cluster start
     start_parser = cluster_subparser.add_parser('start', help='Start a stopped cluster',
-                                             description='Start a stopped cluster')
+                                                description='Start a stopped cluster')
     start_parser.add_argument('identifier', help="cluster's UUID (either long or short) or name", nargs='+')
 
     # tutum cluster stop
     stop_parser = cluster_subparser.add_parser('stop', help='Stop a running cluster',
-                                            description='Stop a running cluster')
+                                               description='Stop a running cluster')
     stop_parser.add_argument('identifier', help="cluster's UUID (either long or short) or name", nargs='+')
 
     # tutum cluster terminate
     terminate_parser = cluster_subparser.add_parser('terminate', help='Terminate an cluster',
-                                                 description='Terminate an cluster')
+                                                    description='Terminate an cluster')
     terminate_parser.add_argument('identifier', help="cluster's UUID (either long or short) or name", nargs='+')
 
 
 def add_container_parser(subparsers):
     # tutum container
     container_parser = subparsers.add_parser('container', help='Container-related operations',
-                                              description='Container-related operations')
+                                             description='Container-related operations')
     container_subparser = container_parser.add_subparsers(title='tutum container commands', dest='subcmd')
 
     # tutum container inspect
     inspect_parser = container_subparser.add_parser('inspect', help='Inspect a container',
-                                                     description='Inspect a container')
+                                                    description='Inspect a container')
     inspect_parser.add_argument('identifier', help="container's UUID (either long or short) or name", nargs='+')
 
     # tutum container logs
     logs_parser = container_subparser.add_parser('logs', help='Get logs from a container',
-                                                  description='Get logs from a container')
+                                                 description='Get logs from a container')
     logs_parser.add_argument('identifier', help="container's UUID (either long or short) or name", nargs='+')
 
     # tutum container ps
@@ -157,20 +158,20 @@ def add_container_parser(subparsers):
 
     # tutum container redeploy
     redeploy_parser = container_subparser.add_parser('redeploy', help='Redeploy a running container with a '
-                                                                 'new version/tag',
-                                                description='Redeploy a running container with a new version/tag')
+                                                                      'new version/tag',
+                                                     description='Redeploy a running container with a new version/tag')
     redeploy_parser.add_argument('identifier', help="container's UUID (either long or short) or name", nargs='+')
     redeploy_parser.add_argument('-t', '--tag', help='tag of the image to redeploy')
 
     # tutum container run
     run_parser = container_subparser.add_parser('run', help='Create and run a new container',
-                                           description='Create and run a new container', )
+                                                description='Create and run a new container', )
     run_parser.add_argument('image', help='the name of the image used to deploy this container')
     run_parser.add_argument('-n', '--name', help='a human-readable name for the container'
                                                  '(default: image_tag without namespace)')
-    run_parser.add_argument('-s', '--container-size', help='the size of the container containers '
-                                                           '(default: XS, possible values: XS, S, M, L, XL)',
-                            default='XS')
+    run_parser.add_argument('--cpushares', help='Relative weight for CPU Shares', type=int)
+    run_parser.add_argument('--memory', help='RAM memory hard limit in MB', type=int)
+    run_parser.add_argument('--memoryswap', help='Memory swap hard limit in MB', type=int)
     run_parser.add_argument('-t', '--target-num-containers',
                             help='the number of containers to run for this container (default: 1)', type=int,
                             default=1)
@@ -212,19 +213,19 @@ def add_container_parser(subparsers):
 
     # tutum container terminate
     terminate_parser = container_subparser.add_parser('terminate', help='Terminate a container',
-                                                       description='Terminate a container')
+                                                      description='Terminate a container')
     terminate_parser.add_argument('identifier', help="container's UUID (either long or short) or name", nargs='+')
 
 
 def add_image_parser(subparsers):
     # tutum image
     image_parser = subparsers.add_parser('image', help='Image-related operations',
-                                          description='Image-related operations')
+                                         description='Image-related operations')
     image_subparser = image_parser.add_subparsers(title='tutum image commands', dest='subcmd')
 
     # tutum image list
     list_parser = image_subparser.add_parser('list', help='List private images',
-                                              description='List private images')
+                                             description='List private images')
     list_parser.add_argument('-q', '--quiet', help='print only image names', action='store_true')
 
     list_exclusive_group = list_parser.add_mutually_exclusive_group()
@@ -233,31 +234,104 @@ def add_image_parser(subparsers):
 
     # tutum image register
     register_parser = image_subparser.add_parser('register',
-                                                  help='Register an image from a private repository in Tutum',
-                                                  description='Register an image from a private repository in Tutum')
+                                                 help='Register an image from a private repository in Tutum',
+                                                 description='Register an image from a private repository in Tutum')
     register_parser.add_argument('image_name', help='full image name, i.e. quay.io/tutum/test-repo')
     register_parser.add_argument('-d', '--description', help='Image description')
 
     # tutum image push
     push_parser = image_subparser.add_parser('push', help='Push a local image to Tutum private registry',
-                                              description='Push a local image to Tutum private registry')
+                                             description='Push a local image to Tutum private registry')
     push_parser.add_argument('name', help='name of the image to push')
     push_parser.add_argument('--public', help='push image to public registry', action='store_true')
 
     # tutum image rm
     rm_parser = image_subparser.add_parser('rm', help='Deregister a private image from Tutum',
-                                            description='Deregister a private image from Tutum')
+                                           description='Deregister a private image from Tutum')
     rm_parser.add_argument('image_name', help='full image name, i.e. quay.io/tutum/test-repo', nargs='+')
 
     # tutum image search
     search_parser = image_subparser.add_parser('search', help='Search for images in the Docker Index',
-                                                description='Search for images in the Docker Index')
+                                               description='Search for images in the Docker Index')
     search_parser.add_argument('query', help='query to search')
 
     # tutum image update
     update_parser = image_subparser.add_parser('update', help='Update a private image',
-                                                description='Update a private image')
+                                               description='Update a private image')
     update_parser.add_argument("image_name", help="full image name, i.e. quay.io/tutum/test-repo", nargs="+")
     update_parser.add_argument('-u', '--username', help='new username to authenticate with the registry')
     update_parser.add_argument('-p', '--password', help='new password to authenticate with the registry')
-    update_parser.add_argument('-d', '--description', help='new image description')
\ No newline at end of file
+    update_parser.add_argument('-d', '--description', help='new image description')
+
+
+def add_node_parser(subparsers):
+    # tutum node
+    node_parser = subparsers.add_parser('node', help='Node-related operations', description='Node-related operations')
+    node_subparser = node_parser.add_subparsers(title='tutum node commands', dest='subcmd')
+
+    # tutum node inspect
+    inspect_parser = node_subparser.add_parser('inspect', help='Inspect a node', description='Inspect a node')
+    inspect_parser.add_argument('identifier', help="node's UUID (either long or short)", nargs='+')
+
+    # tutum node list
+    list_parser = node_subparser.add_parser('list', help='List nodes', description='List nodes')
+    list_parser.add_argument('-q', '--quiet', help='print only node uuid', action='store_true')
+
+    # tutum node rm
+    rm_parser = node_subparser.add_parser('rm', help='Remove a node', description='Remove a container')
+    rm_parser.add_argument('identifier', help="node's UUID (either long or short)", nargs='+')
+
+
+def add_nodecluster_parser(subparsers):
+    # tutum nodecluster
+    nodecluster_parser = subparsers.add_parser('nodecluster', help='NodeCluster-related operations',
+                                               description='NodeCluster-related operations')
+    nodecluster_subparser = nodecluster_parser.add_subparsers(title='tutum node commands', dest='subcmd')
+
+    # tutum nodecluster create
+    create_parser = nodecluster_subparser.add_parser('create', help='Create a nodecluster',
+                                                     description='Create a nodecluster')
+    create_parser.add_argument('-t', '--target-num-nodes',
+                               help='the target number of nodes to run for this cluster (default: 1)', type=int,
+                               default=1)
+    create_parser.add_argument('name', help='name of the cluster to create')
+    create_parser.add_argument('provider_id', help='id of the provider', type=int)
+    create_parser.add_argument('region_id', help='id of the region', type=int)
+    create_parser.add_argument('nodetype_id', help='id of the nodetype', type=int)
+
+    # tutum nodecluster inspect
+    inspect_parser = nodecluster_subparser.add_parser('inspect', help='Inspect a nodecluster',
+                                                      description='Inspect a nodecluster')
+    inspect_parser.add_argument('identifier', help="node's UUID (either long or short)", nargs='+')
+
+    # tutum nodecluster list
+    list_parser = nodecluster_subparser.add_parser('list', help='List node clusters', description='List node clusters')
+    list_parser.add_argument('-q', '--quiet', help='print only node uuid', action='store_true')
+
+    # tutum nodecluster rm
+    rm_parser = nodecluster_subparser.add_parser('rm', help='Remove node clusters', description='Remove node clusters')
+    rm_parser.add_argument('identifier', help="node's UUID (either long or short)", nargs='+')
+
+    # tutum nodecluster scale
+    scale_parser = nodecluster_subparser.add_parser('scale', help='Scale a running node cluster',
+                                                    description='Scale a running node cluster', )
+    scale_parser.add_argument('identifier', help="node cluster's UUID (either long or short) or name", nargs='+')
+    scale_parser.add_argument("target_num_containers", metavar="target-num-containers",
+                              help="target number of nodes to scale this node cluster to", type=int)
+
+    # tutum nodecluster provider
+    provider_parser = nodecluster_subparser.add_parser('provider', help='Show all available infrastructure providers',
+                                                       description='Show all available infrastructure providers')
+    provider_parser.add_argument('-q', '--quiet', help='print only provider name', action='store_true')
+
+    # tutum nodecluster region
+    region_parser = nodecluster_subparser.add_parser('region', help='Show all available regions of a given provider')
+    region_parser.add_argument('provider_id',
+                               help="id of the provider (to find out id, use `tutum nodecluster provider`)",
+                               type=int)
+
+    # tutum nodecluster nodetype
+    nodetype_parser = nodecluster_subparser.add_parser('nodetype', help='Show all available types of a given region')
+    nodetype_parser.add_argument('region_id', help="id of the region (to find out id use `tutum nodecluster region`)",
+                                 type=int)
+
